package com.bsf.lovDemo.components;

import com.jgoodies.swing.application.ResourceManager;
import org.bsf.listOfValues.client.LovManager;
import org.bsf.listOfValues.exceptions.NoSuchLovException;
import org.bsf.listOfValues.exceptions.NoSuchLovValueException;
import org.bsf.listOfValues.lovValue.LovValue;

import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import java.util.Vector;

/**
 * A combobox that handles LovValues as data, retrieves the expected lov,
 * uses the expected field in a LovBoxRenderer and is able to return the
 * oid of the selected LovValue or to select a LovValue when given its
 * oid.
 *
 * @see LovValue
 * @see LovBoxRenderer
 */
public class LovBox extends JComboBox {
    private Long _lovOID = null;
    private static final String LOV_DATA = ResourceManager.getString( "combobox.tooltip.internal" );

    /**
     * @param p_lovOID The oid of the LOV to use for the model.
     * @param p_boxModel The BoxModel to use.
     * @param p_fieldToUse The field to use for the rendering.
     *
     * @throws IllegalArgumentException if the p_fieldToUse is null
     * (cf LovBoxRenderer).
     *
     * @see LovValue
     * @see LovBoxRenderer
     */
    public LovBox( Long p_lovOID, ComboBoxModel p_boxModel, String p_fieldToUse ) {
        super();

        _lovOID = p_lovOID;
        setModel( p_boxModel );
        setRenderer( new LovBoxRenderer( p_fieldToUse ) );

        addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                // Selection has changed... We want to update the tooltip
                updateTooltip();
            }
        } );
    }

    /**
     * @param p_lovOID The oid of the LOV to use for the model.
     * @param p_fieldToUse The field to use for the rendering.
     *
     * @throws NoSuchLovException if no LOV matches the given p_lovOID.
     * @throws RemoteException if a communication problem occurs.
     * @throws IllegalArgumentException if the p_fieldToUse is null
     * (cf LovBoxRenderer).
     *
     * @see LovValue
     * @see LovBoxRenderer
     */
    public LovBox( Long p_lovOID, String p_fieldToUse ) throws NoSuchLovException, RemoteException {
        super( new Vector( LovManager.getListOfValues( p_lovOID ) ) );

        _lovOID = p_lovOID;
        setRenderer( new LovBoxRenderer( p_fieldToUse ) );

        addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                // Selection has changed... We want to update the tooltip
                updateTooltip();
            }
        } );
    }

    public void setModel( ComboBoxModel aModel ) {
        super.setModel( aModel );

        // Selection could have been reseted
        updateTooltip();
    }

    /**
     * @return the PK of the selected LovValue or null if none is selected.
     */
    public Object getSelectedLovValuePK() {
        LovValue selectedLovValue = (LovValue) getSelectedItem();

        return ( selectedLovValue == null ) ? null : selectedLovValue.getPK();
    }

    /**
     * @param p_lovValueOID The oid of the LovValue to select.
     *
     * @throws NoSuchLovValueException if no LovValue, in the actually used LOV, is
     * matching the given LovValueOID.
     */
    public void setSelectedLovValue( Long p_lovValueOID ) throws NoSuchLovValueException {
        try {
            LovValue lovValue = LovManager.getLovValue( _lovOID, p_lovValueOID );

            setSelectedItem( lovValue );
        } catch ( NoSuchLovException e ) {
            // Can't be thrown... We shouldn't be here otherwise (unable to
            // instantiate the combobox in the first place)
        } catch ( RemoteException e ) {
            // Same as above, the LOV should already be in cache at this time
            // In any case we keep the previous selected value...
        }
    }

    /**
     * @return the LovBoxrenderer used by this combobox.
     */
    public LovBoxRenderer getLovBoxRenderer() {
        return (LovBoxRenderer) getRenderer();
    }

    /**
     * Updates the tool tip to display the LovValue content.
     **/
    private void updateTooltip() {
        Object selectedItem = getSelectedItem();

        if ( selectedItem instanceof LovValue ) {
            setToolTipText( LOV_DATA + " " + selectedItem.toString() );
        } else {
            setToolTipText( null );
        }
    }
}
